<html>
    <head>
        <title>Djaq</title>
        <link rel="stylesheet" href="https://cdn.jsdelivr.net/npm/bulma@0.9.1/css/bulma.min.css">
        <script src="https://code.jquery.com/jquery-3.6.0.min.js" integrity="sha256-/xUj+3OJU5yExlq6GSYGSHk7tPXikynS7ogEvDej/m4=" crossorigin="anonymous"></script>
        <style></style>
    </head>
    <body>
        <div style="width:100%;margin:30px;">
            {% block content %}
    {% endblock %}
        </div>
    </body>
</html>
